@@ -41,7 +41,6 @@
 
 #include "cvt-openpgp.h"
 #include "../common/sexp-parse.h"
-#include "../common/openpgpdefs.h"  /* For s2k functions.  */
 
 
 /* The protection mode for encryption.  The supported modes for
@@ -50,6 +49,9 @@
 #define PROT_CIPHER_STRING "aes"
 #define PROT_CIPHER_KEYLEN (128/8)
 
+/* Decode an rfc4880 encoded S2K count.  */
+#define S2K_DECODE_COUNT(_val) ((16ul + ((_val) & 15)) << (((_val) >> 4) + 6))
+
 
 /* A table containing the information needed to create a protected
    private key.  */
@@ -96,36 +98,21 @@ hash_passphrase (const char *passphrase, int hashalgo,
 
 
 
-/*
- * Determine if we can use clock_gettime with CLOCK_THREAD_CPUTIME_ID,
- * at compile time.
- */
-#if defined (CLOCK_THREAD_CPUTIME_ID)
-# if _POSIX_THREAD_CPUTIME > 0
-# define USE_CLOCK_GETTIME 1
-# elif _POSIX_THREAD_CPUTIME == 0
-/*
- * In this case, we should check sysconf with _POSIX_THREAD_CPUTIME at
- * run time.  As heuristics, for system with newer GNU C library, we
- * can assume it is available.
- */
-#  if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 17
-#  define USE_CLOCK_GETTIME 1
-#  endif
-# endif
-#else
-#undef USE_CLOCK_GETTIME
-#endif
-
 /* Get the process time and store it in DATA.  */
 static void
 calibrate_get_time (struct calibrate_time_s *data)
 {
 #ifdef HAVE_W32_SYSTEM
+# ifdef HAVE_W32CE_SYSTEM
+  GetThreadTimes (GetCurrentThread (),
+                  &data->creation_time, &data->exit_time,
+                  &data->kernel_time, &data->user_time);
+# else
   GetProcessTimes (GetCurrentProcess (),
                    &data->creation_time, &data->exit_time,
                    &data->kernel_time, &data->user_time);
-#elif defined (USE_CLOCK_GETTIME)
+# endif
+#elif defined (CLOCK_THREAD_CPUTIME_ID)
   struct timespec tmp;
 
   clock_gettime (CLOCK_THREAD_CPUTIME_ID, &tmp);
@@ -379,11 +366,12 @@ do_encryption (const unsigned char *hashbegin, size_t hashlen,
                const char *passphrase,
                const char *timestamp_exp, size_t timestamp_exp_len,
                unsigned char **result, size_t *resultlen,
-	       unsigned long s2k_count)
+	       unsigned long s2k_count, int use_ocb)
 {
   gcry_cipher_hd_t hd;
   const char *modestr;
-  int enclen, outlen;
+  unsigned char hashvalue[20];
+  int blklen, enclen, outlen;
   unsigned char *iv = NULL;
   unsigned int ivsize;  /* Size of the buffer allocated for IV.  */
   const unsigned char *s2ksalt; /* Points into IV.  */
@@ -397,26 +385,44 @@ do_encryption (const unsigned char *hashbegin, size_t hashlen,
   *resultlen = 0;
   *result = NULL;
 
-  modestr = "openpgp-s2k3-ocb-aes";
+  modestr = (use_ocb? "openpgp-s2k3-ocb-aes"
+             /*   */: "openpgp-s2k3-sha1-" PROT_CIPHER_STRING "-cbc");
 
   rc = gcry_cipher_open (&hd, PROT_CIPHER,
-                         GCRY_CIPHER_MODE_OCB,
+                         use_ocb? GCRY_CIPHER_MODE_OCB :
+                         GCRY_CIPHER_MODE_CBC,
                          GCRY_CIPHER_SECURE);
   if (rc)
     return rc;
 
   /* We need to work on a copy of the data because this makes it
    * easier to add the trailer and the padding and more important we
-   * have to prefix the text with 2 parenthesis.  Due to OCB mode we
+   * have to prefix the text with 2 parenthesis.  In CBC mode we
+   * have to allocate enough space for:
+   *
+   *   ((<parameter_list>)(4:hash4:sha120:<hashvalue>)) + padding
+   *
+   * we always append a full block of random bytes as padding but
+   * encrypt only what is needed for a full blocksize.  In OCB mode we
    * have to allocate enough space for just:
    *
    *   ((<parameter_list>))
    */
-
-  /*       ((            )) */
-  outlen = 2 + protlen + 2 ;
-  enclen = outlen + 16 /* taglen */;
-  outbuf = gcry_malloc_secure (enclen);
+  blklen = gcry_cipher_get_algo_blklen (PROT_CIPHER);
+  if (use_ocb)
+    {
+      /*       ((            )) */
+      outlen = 2 + protlen + 2 ;
+      enclen = outlen + 16 /* taglen */;
+      outbuf = gcry_malloc_secure (enclen);
+    }
+  else
+    {
+      /*       ((            )( 4:hash 4:sha1 20:<hash> ))  <padding>  */
+      outlen = 2 + protlen + 2 + 6   + 6    + 23      + 2 + blklen;
+      enclen = outlen/blklen * blklen;
+      outbuf = gcry_malloc_secure (outlen);
+    }
   if (!outbuf)
     {
       rc = out_of_core ();
@@ -426,10 +432,10 @@ do_encryption (const unsigned char *hashbegin, size_t hashlen,
   /* Allocate a buffer for the nonce and the salt.  */
   if (!rc)
     {
-      /* Allocate random bytes to be used as nonce and s2k salt.  The
-       * nonce is set later because for OCB we need to set the key
-       * first.  */
-      ivsize = 12 + 8;
+      /* Allocate random bytes to be used as IV, padding and s2k salt
+       * or in OCB mode for a nonce and the s2k salt.  The IV/nonce is
+       * set later because for OCB we need to set the key first.  */
+      ivsize = (use_ocb? 12 : (blklen*2)) + 8;
       iv = xtrymalloc (ivsize);
       if (!iv)
         rc = gpg_error_from_syserror ();
@@ -465,17 +471,40 @@ do_encryption (const unsigned char *hashbegin, size_t hashlen,
     goto leave;
 
   /* Set the IV/nonce.  */
-  rc = gcry_cipher_setiv (hd, iv, 12);
+  rc = gcry_cipher_setiv (hd, iv, use_ocb? 12 : blklen);
   if (rc)
     goto leave;
 
-  /* In OCB Mode we use only the public key parameters as AAD.  */
-  rc = gcry_cipher_authenticate (hd, hashbegin, protbegin - hashbegin);
-  if (!rc)
-    rc = gcry_cipher_authenticate (hd, timestamp_exp, timestamp_exp_len);
-  if (!rc)
-    rc = gcry_cipher_authenticate
-      (hd, protbegin+protlen, hashlen - (protbegin+protlen - hashbegin));
+  if (use_ocb)
+    {
+      /* In OCB Mode we use only the public key parameters as AAD.  */
+      rc = gcry_cipher_authenticate (hd, hashbegin, protbegin - hashbegin);
+      if (!rc)
+        rc = gcry_cipher_authenticate (hd, timestamp_exp, timestamp_exp_len);
+      if (!rc)
+        rc = gcry_cipher_authenticate
+          (hd, protbegin+protlen, hashlen - (protbegin+protlen - hashbegin));
+    }
+  else
+    {
+      /* Hash the entire expression for CBC mode.  Because
+       * TIMESTAMP_EXP won't get protected, we can't simply hash a
+       * continuous buffer but need to call md_write several times.  */
+      gcry_md_hd_t md;
+
+      rc = gcry_md_open (&md, GCRY_MD_SHA1, 0 );
+      if (!rc)
+        {
+          gcry_md_write (md, hashbegin, protbegin - hashbegin);
+          gcry_md_write (md, protbegin, protlen);
+          gcry_md_write (md, timestamp_exp, timestamp_exp_len);
+          gcry_md_write (md, protbegin+protlen,
+                         hashlen - (protbegin+protlen - hashbegin));
+          memcpy (hashvalue, gcry_md_read (md, GCRY_MD_SHA1), 20);
+          gcry_md_close (md);
+        }
+    }
+
 
   /* Encrypt.  */
   if (!rc)
@@ -485,15 +514,36 @@ do_encryption (const unsigned char *hashbegin, size_t hashlen,
       *p++ = '(';
       memcpy (p, protbegin, protlen);
       p += protlen;
-      *p++ = ')';
-      *p++ = ')';
-      log_assert ( p - outbuf == outlen);
-      gcry_cipher_final (hd);
-      rc = gcry_cipher_encrypt (hd, outbuf, outlen, NULL, 0);
-      if (!rc)
+      if (use_ocb)
         {
-          log_assert (outlen + 16 == enclen);
-          rc = gcry_cipher_gettag (hd, outbuf + outlen, 16);
+          *p++ = ')';
+          *p++ = ')';
+        }
+      else
+        {
+          memcpy (p, ")(4:hash4:sha120:", 17);
+          p += 17;
+          memcpy (p, hashvalue, 20);
+          p += 20;
+          *p++ = ')';
+          *p++ = ')';
+          memcpy (p, iv+blklen, blklen); /* Add padding.  */
+          p += blklen;
+        }
+      assert ( p - outbuf == outlen);
+      if (use_ocb)
+        {
+          gcry_cipher_final (hd);
+          rc = gcry_cipher_encrypt (hd, outbuf, outlen, NULL, 0);
+          if (!rc)
+            {
+              log_assert (outlen + 16 == enclen);
+              rc = gcry_cipher_gettag (hd, outbuf + outlen, 16);
+            }
+        }
+      else
+        {
+          rc = gcry_cipher_encrypt (hd, outbuf, enclen, NULL, 0);
         }
     }
 
@@ -521,7 +571,7 @@ do_encryption (const unsigned char *hashbegin, size_t hashlen,
        (int)strlen (modestr), modestr,
        &saltpos,
        (unsigned int)strlen (countbuf), countbuf,
-       12, &ivpos, 12, "",
+       use_ocb? 12 : blklen, &ivpos, use_ocb? 12 : blklen, "",
        enclen, &encpos, enclen, "");
     if (!p)
       {
@@ -535,7 +585,7 @@ do_encryption (const unsigned char *hashbegin, size_t hashlen,
   *resultlen = strlen (p);
   *result = (unsigned char*)p;
   memcpy (p+saltpos, s2ksalt, 8);
-  memcpy (p+ivpos, iv, 12);
+  memcpy (p+ivpos, iv, use_ocb? 12 : blklen);
   memcpy (p+encpos, outbuf, enclen);
   xfree (iv);
   xfree (outbuf);
@@ -551,11 +601,13 @@ do_encryption (const unsigned char *hashbegin, size_t hashlen,
 
 
 /* Protect the key encoded in canonical format in PLAINKEY.  We assume
- * a valid S-Exp here.  */
+   a valid S-Exp here.  With USE_UCB set to -1 the default scheme is
+   used (ie. either CBC or OCB), set to 0 the old CBC mode is used,
+   and set to 1 OCB is used. */
 int
 agent_protect (const unsigned char *plainkey, const char *passphrase,
                unsigned char **result, size_t *resultlen,
-	       unsigned long s2k_count)
+	       unsigned long s2k_count, int use_ocb)
 {
   int rc;
   const char *parmlist;
@@ -572,6 +624,9 @@ agent_protect (const unsigned char *plainkey, const char *passphrase,
   unsigned char *p;
   int have_curve = 0;
 
+  if (use_ocb == -1)
+    use_ocb = !!opt.enable_extended_key_format;
+
   /* Create an S-expression with the protected-at timestamp.  */
   memcpy (timestamp_exp, "(12:protected-at15:", 19);
   gnupg_get_isotime (timestamp_exp+19);
@@ -665,17 +720,17 @@ agent_protect (const unsigned char *plainkey, const char *passphrase,
   hash_end = s;
   s++;
   /* Skip to the end of the S-expression.  */
-  log_assert (depth == 1);
+  assert (depth == 1);
   rc = sskip (&s, &depth);
   if (rc)
     return rc;
-  log_assert (!depth);
+  assert (!depth);
   real_end = s-1;
 
   rc = do_encryption (hash_begin, hash_end - hash_begin + 1,
                       prot_begin, prot_end - prot_begin + 1,
                       passphrase, timestamp_exp, sizeof (timestamp_exp),
-                      &protected, &protectedlen, s2k_count);
+                      &protected, &protectedlen, s2k_count, use_ocb);
   if (rc)
     return rc;
 
@@ -707,7 +762,7 @@ agent_protect (const unsigned char *plainkey, const char *passphrase,
 
   memcpy (p, prot_end+1, real_end - prot_end);
   p += real_end - prot_end;
-  log_assert ( p - *result == *resultlen);
+  assert ( p - *result == *resultlen);
   xfree (protected);
 
   return 0;
@@ -896,10 +951,7 @@ merge_lists (const unsigned char *protectedkey,
   /* Copy the cleartext.  */
   s = cleartext;
   if (*s != '(' && s[1] != '(')
-    {
-      xfree (newlist);
-      return gpg_error (GPG_ERR_BUG);  /*we already checked this */
-    }
+    return gpg_error (GPG_ERR_BUG);  /*we already checked this */
   s += 2;
   startpos = s;
   while ( *s == '(' )
@@ -949,7 +1001,7 @@ merge_lists (const unsigned char *protectedkey,
 
   /* Skip over the protected list element in the original list.  */
   s = protectedkey + replacepos;
-  log_assert (*s == '(');
+  assert (*s == '(');
   s++;
   i = 1;
   rc = sskip (&s, &i);
@@ -976,7 +1028,7 @@ merge_lists (const unsigned char *protectedkey,
   rc = sskip (&s, &i);
   if (rc)
     goto failure;
-  log_assert (s[-1] == ')');
+  assert (s[-1] == ')');
   endpos = s; /* one behind the end of the list */
 
   /* Append the rest. */
@@ -1076,7 +1128,7 @@ agent_unprotect (ctrl_t ctrl,
   if (!protect_info[infidx].algo)
     return gpg_error (GPG_ERR_UNSUPPORTED_ALGORITHM);
 
-  /* See whether we have a protected-at timestamp.  */
+  /* See wether we have a protected-at timestamp.  */
   protect_list = s;  /* Save for later.  */
   if (protected_at)
     {
@@ -1466,10 +1518,9 @@ make_shadow_info (const char *serialno, const char *idstring)
   to. The input parameters are expected to be valid canonicalized
   S-expressions */
 int
-agent_shadow_key_type (const unsigned char *pubkey,
-                       const unsigned char *shadow_info,
-                       const unsigned char *type,
-                       unsigned char **result)
+agent_shadow_key (const unsigned char *pubkey,
+                  const unsigned char *shadow_info,
+                  unsigned char **result)
 {
   const unsigned char *s;
   const unsigned char *point;
@@ -1522,10 +1573,10 @@ agent_shadow_key_type (const unsigned char *pubkey,
   point = s; /* insert right before the point */
   depth--;
   s++;
-  log_assert (depth == 1);
+  assert (depth == 1);
 
   /* Calculate required length by taking in account: the "shadowed-"
-     prefix, the "shadowed", shadow type as well as some parenthesis */
+     prefix, the "shadowed", "t1-v1" as well as some parenthesis */
   n = 12 + pubkey_len + 1 + 3+8 + 2+5 + shadow_info_len + 1;
   *result = xtrymalloc (n);
   p = (char*)*result;
@@ -1535,7 +1586,7 @@ agent_shadow_key_type (const unsigned char *pubkey,
   /* (10:public-key ...)*/
   memcpy (p, pubkey+14, point - (pubkey+14));
   p += point - (pubkey+14);
-  p += sprintf (p, "(8:shadowed%d:%s", (int)strlen(type), type);
+  p = stpcpy (p, "(8:shadowed5:t1-v1");
   memcpy (p, shadow_info, shadow_info_len);
   p += shadow_info_len;
   *p++ = ')';
@@ -1545,26 +1596,16 @@ agent_shadow_key_type (const unsigned char *pubkey,
   return 0;
 }
 
-int
-agent_shadow_key (const unsigned char *pubkey,
-                  const unsigned char *shadow_info,
-                  unsigned char **result)
-{
-  return agent_shadow_key_type (pubkey, shadow_info, "t1-v1", result);
-}
-
 /* Parse a canonical encoded shadowed key and return a pointer to the
-   inner list with the shadow_info and the shadow type */
+   inner list with the shadow_info */
 gpg_error_t
-agent_get_shadow_info_type (const unsigned char *shadowkey,
-                            unsigned char const **shadow_info,
-                            unsigned char **shadow_type)
+agent_get_shadow_info (const unsigned char *shadowkey,
+                       unsigned char const **shadow_info)
 {
-  const unsigned char *s, *saved_s;
-  size_t n, saved_n;
+  const unsigned char *s;
+  size_t n;
   int depth = 0;
 
-  (void)depth;
   s = shadowkey;
   if (*s != '(')
     return gpg_error (GPG_ERR_INV_SEXP);
@@ -1611,80 +1652,24 @@ agent_get_shadow_info_type (const unsigned char *shadowkey,
   n = snext (&s);
   if (!n)
     return gpg_error (GPG_ERR_INV_SEXP);
-  saved_s = s;
-  saved_n = n;
-  if (smatch (&s, n, "t1-v1") || smatch(&s, n, "tpm2-v1"))
+  if (smatch (&s, n, "t1-v1"))
     {
       if (*s != '(')
         return gpg_error (GPG_ERR_INV_SEXP);
-      if (shadow_info)
-        *shadow_info = s;
+      *shadow_info = s;
     }
   else
     return gpg_error (GPG_ERR_UNSUPPORTED_PROTOCOL);
-  s = saved_s;
-  n = saved_n;
-
-  if (shadow_type)
-    {
-      char *buf = xtrymalloc(n+1);
-      if (!buf)
-        return gpg_error_from_syserror ();
-      memcpy (buf, s, n);
-      buf[n] = '\0';
-      *shadow_type = buf;
-    }
-
   return 0;
 }
 
 
-gpg_error_t
-agent_get_shadow_info (const unsigned char *shadowkey,
-                       unsigned char const **shadow_info)
-{
-  return agent_get_shadow_info_type (shadowkey, shadow_info, NULL);
-}
-
-
-int
-agent_is_tpm2_key (gcry_sexp_t s_skey)
-{
-  unsigned char *buf;
-  unsigned char *type;
-  size_t len;
-  gpg_error_t err;
-
-  err = make_canon_sexp (s_skey, &buf, &len);
-  if (err)
-    return 0;
-
-  err = agent_get_shadow_info_type (buf, NULL, &type);
-  xfree (buf);
-  if (err)
-    return 0;
-
-  err = strcmp (type, "tpm2-v1") == 0;
-  xfree (type);
-  return err;
-}
-
-
-gpg_error_t
-agent_get_shadow_type (const unsigned char *shadowkey,
-                       unsigned char **shadow_type)
-{
-  return agent_get_shadow_info_type (shadowkey, NULL, shadow_type);
-}
-
-
 /* Parse the canonical encoded SHADOW_INFO S-expression.  On success
    the hex encoded serial number is returned as a malloced strings at
    R_HEXSN and the Id string as a malloced string at R_IDSTR.  On
    error an error code is returned and NULL is stored at the result
    parameters addresses.  If the serial number or the ID string is not
-   required, NULL may be passed for them.  Note that R_PINLEN is
-   currently not used by any caller.  */
+   required, NULL may be passed for them.  */
 gpg_error_t
 parse_shadow_info (const unsigned char *shadow_info,
                    char **r_hexsn, char **r_idstr, int *r_pinlen)
