@@ -118,6 +118,7 @@ struct close_cache_s
 typedef struct close_cache_s *close_cache_t;
 static close_cache_t close_cache;
 
+int iobuf_debug_mode;
 
 
 #ifdef HAVE_W32_SYSTEM
@@ -2216,6 +2217,7 @@ iobuf_copy (iobuf_t dest, iobuf_t source)
 
   size_t nread;
   size_t nwrote = 0;
+  size_t max_read = 0;
   int err;
 
   assert (source->use == IOBUF_INPUT || source->use == IOBUF_INPUT_TEMP);
@@ -2232,6 +2234,9 @@ iobuf_copy (iobuf_t dest, iobuf_t source)
         /* EOF.  */
         break;
 
+      if (nread > max_read)
+        max_read = nread;
+
       err = iobuf_write (dest, temp, nread);
       if (err)
         break;
@@ -2239,7 +2244,8 @@ iobuf_copy (iobuf_t dest, iobuf_t source)
     }
 
   /* Burn the buffer.  */
-  wipememory (temp, sizeof (temp));
+  if (max_read)
+    wipememory (temp, max_read);
   xfree (temp);
 
   return nwrote;
@@ -2329,7 +2335,7 @@ iobuf_get_filelength (iobuf_t a, int *overflow)
 	  return size;
       }
     log_error ("GetFileSize for handle %p failed: %s\n",
-	       fp, w32_strerror (0));
+	       fp, w32_strerror (-1));
 #else /*!HAVE_W32_SYSTEM*/
     {
       struct stat st;
