@@ -25,6 +25,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <time.h>
+#include <assert.h>
 #include <ctype.h>
 
 #include "gpgsm.h"
@@ -64,8 +65,6 @@ struct isvalid_status_parm_s {
   ctrl_t ctrl;
   int seen;
   unsigned char fpr[20];
-  gnupg_isotime_t revoked_at;
-  char *revocation_reason;  /* malloced or NULL */
 };
 
 
@@ -157,9 +156,39 @@ static gpg_error_t
 warn_version_mismatch (ctrl_t ctrl, assuan_context_t ctx,
                        const char *servername, int mode)
 {
-  return warn_server_version_mismatch (ctx, servername, mode,
-                                       gpgsm_status2, ctrl,
-                                       !opt.quiet);
+  gpg_error_t err;
+  char *serverversion;
+  const char *myversion = strusage (13);
+
+  err = get_assuan_server_version (ctx, mode, &serverversion);
+  if (err)
+    log_error (_("error getting version from '%s': %s\n"),
+               servername, gpg_strerror (err));
+  else if (compare_version_strings (serverversion, myversion) < 0)
+    {
+      char *warn;
+
+      warn = xtryasprintf (_("server '%s' is older than us (%s < %s)"),
+                           servername, serverversion, myversion);
+      if (!warn)
+        err = gpg_error_from_syserror ();
+      else
+        {
+          log_info (_("WARNING: %s\n"), warn);
+          if (!opt.quiet)
+            {
+              log_info (_("Note: Outdated servers may lack important"
+                          " security fixes.\n"));
+              log_info (_("Note: Use the command \"%s\" to restart them.\n"),
+                        "gpgconf --kill all");
+            }
+          gpgsm_status2 (ctrl, STATUS_WARNING, "server_version_mismatch 0",
+                         warn, NULL);
+          xfree (warn);
+        }
+    }
+  xfree (serverversion);
+  return err;
 }
 
 
@@ -169,7 +198,7 @@ warn_version_mismatch (ctrl_t ctrl, assuan_context_t ctx,
 static void
 prepare_dirmngr (ctrl_t ctrl, assuan_context_t ctx, gpg_error_t err)
 {
-  strlist_t server;
+  struct keyserver_spec *server;
 
   if (!err)
     err = warn_version_mismatch (ctrl, ctx, DIRMNGR_NAME, 0);
@@ -190,13 +219,12 @@ prepare_dirmngr (ctrl_t ctrl, assuan_context_t ctx, gpg_error_t err)
   while (server)
     {
       char line[ASSUAN_LINELENGTH];
+      char *user = server->user ? server->user : "";
+      char *pass = server->pass ? server->pass : "";
+      char *base = server->base ? server->base : "";
 
-      /* If the host is "ldap" we prefix the entire line with "ldap:"
-       * to avoid an ambiguity on the server due to the introduction
-       * of this optional prefix.  */
-      snprintf (line, DIM (line), "LDAPSERVER %s%s",
-                !strncmp (server->d, "ldap:", 5)? "ldap:":"",
-                server->d);
+      snprintf (line, DIM (line), "LDAPSERVER %s:%i:%s:%s:%s",
+		server->host, server->port, user, pass, base);
 
       assuan_transact (ctx, line, NULL, NULL, NULL, NULL, NULL, NULL);
       /* The code below is not required because we don't return an error.  */
@@ -254,7 +282,7 @@ start_dirmngr (ctrl_t ctrl)
 {
   gpg_error_t err;
 
-  log_assert (! dirmngr_ctx_locked);
+  assert (! dirmngr_ctx_locked);
   dirmngr_ctx_locked = 1;
 
   err = start_dirmngr_ext (ctrl, &dirmngr_ctx);
@@ -284,7 +312,7 @@ start_dirmngr2 (ctrl_t ctrl)
 {
   gpg_error_t err;
 
-  log_assert (! dirmngr2_ctx_locked);
+  assert (! dirmngr2_ctx_locked);
   dirmngr2_ctx_locked = 1;
 
   err = start_dirmngr_ext (ctrl, &dirmngr2_ctx);
@@ -386,8 +414,8 @@ inq_certificate (void *opaque, const char *line)
       int err;
       ksba_cert_t cert;
 
-      err = gpgsm_find_cert (parm->ctrl, line, ski, &cert,
-                             FIND_CERT_ALLOW_AMBIG|FIND_CERT_WITH_EPHEM);
+
+      err = gpgsm_find_cert (parm->ctrl, line, ski, &cert, 1);
       if (err)
         {
           log_error ("certificate not found: %s\n", gpg_strerror (err));
@@ -427,51 +455,6 @@ unhexify_fpr (const char *hexstr, unsigned char *fpr)
 }
 
 
-/* This is a helper to print diagnostics from dirmngr indicated by
- * WARNING or NOTE status lines.  Returns true if the status LINE was
- * processed.  */
-static int
-warning_and_note_printer (const char *line)
-{
-  const char *s, *s2;
-  const char *warn = NULL;
-  int is_note = 0;
-
-  if ((s = has_leading_keyword (line, "WARNING")))
-    ;
-  else if ((is_note = !!(s = has_leading_keyword (line, "NOTE"))))
-    ;
-  else
-    return 0;  /* Nothing to process.  */
-
-  if ((s2 = has_leading_keyword (s, "no_crl_due_to_tor"))
-      || (s2 = has_leading_keyword (s, "no_ldap_due_to_tor"))
-      || (s2 = has_leading_keyword (s, "no_ocsp_due_to_tor")))
-    warn = _("Tor might be in use - network access is limited");
-  else
-    warn = NULL;
-
-  if (warn)
-    {
-      if (is_note)
-        log_info (_("Note: %s\n"), warn);
-      else
-        log_info (_("WARNING: %s\n"), warn);
-      if (s2)
-        {
-          while (*s2 && !spacep (s2))
-            s2++;
-          while (*s2 && spacep (s2))
-            s2++;
-          if (*s2)
-            gpgsm_print_further_info ("%s", s2);
-        }
-    }
-
-  return 1;  /* Status line processed.  */
-}
-
-
 static gpg_error_t
 isvalid_status_cb (void *opaque, const char *line)
 {
@@ -491,25 +474,8 @@ isvalid_status_cb (void *opaque, const char *line)
     {
       parm->seen++;
       if (!*s || !unhexify_fpr (s, parm->fpr))
-        parm->seen++; /* Bump it to indicate an error. */
+        parm->seen++; /* Bumb it to indicate an error. */
     }
-  else if ((s = has_leading_keyword (line, "REVOCATIONINFO")))
-    {
-      if (*s && strlen (s) >= 15)
-        {
-          memcpy (parm->revoked_at, s, 15);
-          parm->revoked_at[15] = 0;
-        }
-      s += 15;
-      while (*s && spacep (s))
-        s++;
-      xfree (parm->revocation_reason);
-      parm->revocation_reason = *s? xtrystrdup (s) : NULL;
-    }
-  else if (warning_and_note_printer (line))
-    {
-    }
-
   return 0;
 }
 
@@ -521,22 +487,16 @@ isvalid_status_cb (void *opaque, const char *line)
 
   GPG_ERR_CERTIFICATE_REVOKED
   GPG_ERR_NO_CRL_KNOWN
-  GPG_ERR_INV_CRL_OBJ
   GPG_ERR_CRL_TOO_OLD
 
   Values for USE_OCSP:
      0 = Do CRL check.
-     1 = Do an OCSP check but fallback to CRL unless CRLs are disabled.
-     2 = Do only an OCSP check (used for the chain model).
-
-   If R_REVOKED_AT pr R_REASON are not NULL and the certificate has
-   been revoked the revocation time and the reason are copied to there.
-   The caller needs to free R_REASON.
+     1 = Do an OCSP check but fallback to CRL unless CRLS are disabled.
+     2 = Do only an OCSP check using only the default responder.
  */
-gpg_error_t
+int
 gpgsm_dirmngr_isvalid (ctrl_t ctrl,
-                       ksba_cert_t cert, ksba_cert_t issuer_cert, int use_ocsp,
-                       gnupg_isotime_t r_revoked_at, char **r_reason)
+                       ksba_cert_t cert, ksba_cert_t issuer_cert, int use_ocsp)
 {
   static int did_options;
   int rc;
@@ -545,11 +505,6 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
   struct inq_certificate_parm_s parm;
   struct isvalid_status_parm_s stparm;
 
-  if (r_revoked_at)
-    *r_revoked_at = 0;
-  if (r_reason)
-    *r_reason = NULL;
-
   rc = start_dirmngr (ctrl);
   if (rc)
     return rc;
@@ -579,8 +534,6 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
   stparm.ctrl = ctrl;
   stparm.seen = 0;
   memset (stparm.fpr, 0, 20);
-  stparm.revoked_at[0] = 0;
-  stparm.revocation_reason = NULL;
 
   /* It is sufficient to send the options only once because we have
    * one connection per process only.  */
@@ -591,8 +544,9 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
                          NULL, NULL, NULL, NULL, NULL, NULL);
       did_options = 1;
     }
-  snprintf (line, DIM(line), "ISVALID%s %s%s%s",
-            (use_ocsp == 2 || opt.no_crl_check) ? " --only-ocsp":"",
+  snprintf (line, DIM(line), "ISVALID%s%s %s%s%s",
+            use_ocsp == 2 || opt.no_crl_check ? " --only-ocsp":"",
+            use_ocsp == 2? " --force-default-responder":"",
             certid,
             use_ocsp? " ":"",
             use_ocsp? certfpr:"");
@@ -605,19 +559,6 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
   if (opt.verbose > 1)
     log_info ("response of dirmngr: %s\n", rc? gpg_strerror (rc): "okay");
 
-  if (gpg_err_code (rc) == GPG_ERR_CERT_REVOKED
-      && !check_isotime (stparm.revoked_at))
-    {
-      if (r_revoked_at)
-        gnupg_copy_time (r_revoked_at, stparm.revoked_at);
-      if (r_reason)
-        {
-          *r_reason = stparm.revocation_reason;
-          stparm.revocation_reason = NULL;
-        }
-
-    }
-
   if (!rc && stparm.seen)
     {
       /* Need to also check the certificate validity. */
@@ -636,7 +577,7 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
                  from the dirmngr.  Try our own cert store now.  */
               KEYDB_HANDLE kh;
 
-              kh = keydb_new (ctrl);
+              kh = keydb_new ();
               if (!kh)
                 rc = gpg_error (GPG_ERR_ENOMEM);
               if (!rc)
@@ -661,8 +602,7 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
                 {
                   /* Note the no_dirmngr flag: This avoids checking
                      this certificate over and over again. */
-                  rc = gpgsm_validate_chain (ctrl, rspcert, GNUPG_ISOTIME_NONE,
-                                             NULL, 0, NULL,
+                  rc = gpgsm_validate_chain (ctrl, rspcert, "", NULL, 0, NULL,
                                              VALIDATE_FLAG_NO_DIRMNGR, NULL);
                   if (rc)
                     {
@@ -675,9 +615,7 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
           ksba_cert_release (rspcert);
         }
     }
-
   release_dirmngr (ctrl);
-  xfree (stparm.revocation_reason);
   return rc;
 }
 
@@ -813,10 +751,6 @@ lookup_status_cb (void *opaque, const char *line)
           gpgsm_status (parm->ctrl, STATUS_TRUNCATED, line);
         }
     }
-  else if (warning_and_note_printer (line))
-    {
-    }
-
   return 0;
 }
 
@@ -1001,48 +935,21 @@ static gpg_error_t
 run_command_inq_cb (void *opaque, const char *line)
 {
   struct run_command_parm_s *parm = opaque;
-  gpg_error_t err;
   const char *s;
   int rc = 0;
-  ksba_cert_t cert = NULL;
-  ksba_sexp_t ski = NULL;
-  const unsigned char *der;
-  size_t derlen, n;
 
   if ((s = has_leading_keyword (line, "SENDCERT")))
-    {
-      /* Send the given certificate.  */
+    { /* send the given certificate */
+      int err;
+      ksba_cert_t cert;
+      const unsigned char *der;
+      size_t derlen;
+
       line = s;
       if (!*line)
         return gpg_error (GPG_ERR_ASS_PARAMETER);
 
-      err = gpgsm_find_cert (parm->ctrl, line, NULL, &cert,
-                             FIND_CERT_ALLOW_AMBIG);
-      if (err)
-        {
-          log_error ("certificate not found: %s\n", gpg_strerror (err));
-          rc = gpg_error (GPG_ERR_NOT_FOUND);
-        }
-      else
-        {
-          der = ksba_cert_get_image (cert, &derlen);
-          if (!der)
-            rc = gpg_error (GPG_ERR_INV_CERT_OBJ);
-          else
-            rc = assuan_send_data (parm->ctx, der, derlen);
-        }
-    }
-  else if ((s = has_leading_keyword (line, "SENDCERT_SKI")))
-    {
-      /* Send a certificate where a sourceKeyIdentifier is included. */
-      line = s;
-      ski = make_simple_sexp_from_hexstr (line, &n);
-      line += n;
-      while (*line == ' ')
-        line++;
-
-      err = gpgsm_find_cert (parm->ctrl, line, ski, &cert,
-                             FIND_CERT_ALLOW_AMBIG|FIND_CERT_WITH_EPHEM);
+      err = gpgsm_find_cert (parm->ctrl, line, NULL, &cert, 1);
       if (err)
         {
           log_error ("certificate not found: %s\n", gpg_strerror (err));
@@ -1055,11 +962,11 @@ run_command_inq_cb (void *opaque, const char *line)
             rc = gpg_error (GPG_ERR_INV_CERT_OBJ);
           else
             rc = assuan_send_data (parm->ctx, der, derlen);
+          ksba_cert_release (cert);
         }
     }
   else if ((s = has_leading_keyword (line, "PRINTINFO")))
-    {
-      /* Simply show the message given in the argument. */
+    { /* Simply show the message given in the argument. */
       line = s;
       log_info ("dirmngr: %s\n", line);
     }
@@ -1069,6 +976,7 @@ run_command_inq_cb (void *opaque, const char *line)
          root certificate.  */
       char fpr[41];
       struct rootca_flags_s rootca_flags;
+      int n;
 
       line = s;
 
@@ -1092,8 +1000,6 @@ run_command_inq_cb (void *opaque, const char *line)
       rc = gpg_error (GPG_ERR_ASS_UNKNOWN_INQUIRE);
     }
 
-  ksba_cert_release (cert);
-  xfree (ski);
   return rc;
 }
 
@@ -1116,10 +1022,6 @@ run_command_status_cb (void *opaque, const char *line)
             return gpg_error (GPG_ERR_ASS_CANCELED);
         }
     }
-  else if (warning_and_note_printer (line))
-    {
-    }
-
   return 0;
 }
 
